# frozen_string_literal: true

module Repubmark
  module Elems
    class Note < Base
      parents :Joint

      UNICODE_SUPS = {
        '1' => '¹',
        '2' => '²',
        '3' => '³',
        '4' => '⁴',
        '5' => '⁵',
        '6' => '⁶',
        '7' => '⁷',
        '8' => '⁸',
        '9' => '⁹',
        '0' => '⁰',
      }.freeze

      def initialize(parent, index, anchor)
        super parent
        @index = index
        @anchor = anchor
      end

      #################
      # Basic methods #
      #################

      def to_html
        [
          '<sup>',
          %(<a href="##{CGI.escape_html(@anchor)}">),
          %([#{CGI.escape_html(@index.to_s)}]),
          '</a>',
          '</sup>',
        ].join.freeze
      end

      def to_gemtext = "⁽#{index_unicode_sup}⁾".freeze

    private

      def index_unicode_sup
        @index_unicode_sup ||=
          @index.to_s.each_char.map { |chr| UNICODE_SUPS.fetch chr }.join.freeze
      end
    end
  end
end
